clear all; clc; close all;
disp('  '); disp(  '    FIRST ASSIGNMENT: VARs'); disp(' ');

% This program identifies fed fund shocks and computes the
% associated impulse responses in the VARs considered by Christiano,
% Eichenbaum, Vigfusson (CEV, NBER Working Paper 9819), and Altig, Christiano, Eichenbaum, Linde
% (ACEL).


%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
%   Paramaters controlling simulation      %
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%

cev         = 0;                % Indicator that VAR is same as Christiano, Eichenbaum, Vigfusson [d(Y/H) H dP C/Y I/Y FF]
acel        = (cev==0);         % Indicator that VAR is same as Altig, Christiano, Eichenbaum, Linde. [d(Y/H) H dP C/Y I/Y U (Y/H)/W FF V]

FFShock     = 1;                % Indicator, include FF shock
TTShock     = 0;                % Indicator, include neutral technology shock
ISShock     = 0;                % Indicator, include investment-specific technology shock (if zero, don't include change in PI in VAR)
%note: you cannot have only the neutral shock
HOURSLEVELS = 0;                % Indicator, hours in level (=0 hours in first difference)
trendval    = [];               % Order of the polynomial used to detrend the data (='[]' no detrending)
vardetrend  = [];               % Indices of the variables detrended (ex.: [1 3 4 7], [] if no detrending)
hascon      = 1;                % Indicator, constant in the VAR
beg_samp    = 45;               % beginning of sample period in 59Q1 (45) (from 48Q1(1) to 01Q4(216), with minimum 54Q3(27))
%beg_samp    = 141;              % beginning in 83Q1(141)
nlag        = 4;                % Number of lags in the VAR
%nlag        = 2;
nstep       = 20;               % Length of impulse responses computed
pnstep      = nstep;            % Length of impulse responses plotted
ndraws      = 200;              % Number of simulated series

% Note: The default ordering is:
% CEV:  [d(Y/H) H dP C/Y I/Y FF]
% ACEL: [d(Y/H) H dP C/Y I/Y U (Y/H)/W FF V]
% To change this order, enter the indices of the columns in the order of
% their appearance in the VAR.

order_cev   = [];               % default order 
order_acel  = [];               % default order 
%order_cev   = [3 6 1 2 4 5];    % [dP FF d(Y/H) H C/Y I/Y]
%order_acel  = [3 8 1 2 4:7 9];  % [dP FF d(Y/H) H C/Y I/Y U Y/H/W V]



if (FFShock==0 && TTShock==0 && ISShock==0), error('Must enter at least one shock to be identified'); end
if (cev==1 && acel==1), error('Cannot choose cev AND acel (mainFF)'); end
if (nstep<pnstep), error('Length of IRF displayed longer than IRF computed (nstep<pstep'); end

% Write labels and order of variables for FF reordering

if cev==1
    if isempty(order_cev); order_cev=1:6; end
    if length(order_cev)~=6; error('Wrong length of order_cev (mainFF'); end
    order=order_cev; ff_defloc=6;
    namz = char('Output','Hours','Inflation','Fed Funds','Consumption','Investment');
else
    if isempty(order_acel); order_acel=1:9; end
    if length(order_acel)~=9; error('Wrong length of order_acel (mainFF'); end
    order=order_acel; ff_defloc=8;
    namz = char('Output','Hours','Inflation','Cap Util','Consumption','Investment','Wages','Fed Funds','Velocity','Money Gr.');
end



%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
%           Loading the data               %
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%

% Loading raw data

load main4data;
load invprice;

% Scaling raw data

YRZ = str2num(yrz(5:size(yrz,1),:));
percaphours =  LBMN./P16;           % Business (including farming) hours worked per capita
pgdp = GDP./GDPQ;                   % GDP deflator
ymm = (GDP./pgdp)./P16 ;            % Real, chain-weighted, output
cmm = ((GCN+GCS+GGE)./pgdp)./P16;   % Consumption on nondurables and services and government
imm = ((GCD + GPI)./pgdp)./P16 ;    % Consumption of durables and private investment. 
cmmratio = (GCN+GCS+GGE)./GDP;      % Consumption on nondurables and services and government to GDP ratio
immratio = (GCD + GPI)./GDP;        % Consumption of durables and private investment to GDP ratio
wagez = exp(log(LBCPU)-log(pgdp));  % Real Wage
capu = IPXMCA;                      % Capacity utilization

% Creating data for VAR

% Note: All variables, except FYFF, are created with the same length. They
% can be passed directly in the VAR without readjusting the length due to
% differencing.

disp('Loading and creating data');

% Log per capita hours (Level or FD)
if HOURSLEVELS == 1, hours = log(percaphours(2:end,1)); sortprog = 'sortlpvar';
else hours = diff(log(percaphours)); sortprog = 'sortlpdhvar'; end

FDL_lbout   =   diff(log(LBOUT));           % FD in log business labor productivity
FDL_pgdp    =   diff(log(pgdp));            % FD in log GDP deflator
L_cmmratio  =   log(cmmratio(2:end));       % Log of C/Y
L_immratio  =   log(immratio(2:end));       % Log of durables and private investment to GDP
L_capu      =   log(capu(2:end));           % Log capital utilization
L_wages     =   log(ymm(2:end)./percaphours(2:end))-log(wagez(2:end));  % Log wage
velocity    =   (GDP(2:end)./FM2(2:end));   % velocity
L_velocity  =   log(velocity);

FDL_pi      =   diff(log(p_i(5:length(p_i)))); % First difference in price of investment

% Putting data in matrix for var

if cev==1, xdata = [FDL_lbout hours FDL_pgdp L_cmmratio L_immratio FYFF(2:end)];
else xdata = [FDL_lbout hours FDL_pgdp L_cmmratio L_immratio L_capu L_wages FYFF(2:end) L_velocity]; end
if ISShock==1
    xdata=[FDL_pi xdata]; 
    ff_defloc=ff_defloc+1; 
    namz = strvcat(namz,'Inv Price');
end 
nvars = size(xdata,2);

% Reordering the VAR if FF shock is ONLY shock (TTShock=0 and IShock=0)

if (TTShock==0 && ISShock==0) % ONLY FF shock, reordering allowed
    xdata = xdata(:,order); 
    mat_order=[1:length(order); order];
    mat_order=sortrows(mat_order',2)';
    fflocation=mat_order(1,ff_defloc);
    sortprog = inline( [sortprog '(x(:,[' int2str(mat_order(1,:)) ']))'],'x');
elseif (or(TTShock,ISShock) && FFShock==0) % LR restriction estimated, no reordering allowed
    fflocation=[]; % No fed fund shock estimated
else
    fflocation=ff_defloc; % Estimate tech shock AND ff shock
end

if (TTShock==1 && ISShock==0 && FFShock==0)
    error('fatal: program is not written to handle the neutral shock alone')    
end


%~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
%            VAR             %
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~%

% Estimating VAR, recovering IRF, shocks, variance decompositions

disp('Estimating VAR');
if or(ISShock,TTShock) 
    [impzout,vdout,simdataout,erzout,azeroout,a0betazout,var_explode] = mkimplr(xdata(beg_samp:end,:),nlag,nstep,hascon,trendval,vardetrend,fflocation,sortprog,ISShock);
elseif FFShock == 1
    [impzout,vdout,simdataout,erzout,azeroout,a0betazout,var_explode] = mkimprec(xdata(beg_samp:end,:),nlag,nstep,hascon,trendval,vardetrend,fflocation,sortprog,ISShock);
end

% Computing confidence intervals on impulse responses

disp(['Generating ' int2str(ndraws) ' simulated series']); disp(' ');
randn('state',32004); rand('state',40032);  % Set pseudo-random generator initial values
simimpzout = zeros([size(impzout) ndraws]); % Create matrix to store simulated IRF
simvdout  = zeros([size(vdout) ndraws]);    % Create matrix to store simulated variance decompositions

if or(ISShock,TTShock)
    simdout = simdata(xdata(beg_samp:end,:),nlag,nstep,hascon,trendval,vardetrend,fflocation,sortprog,ndraws,ISShock);
else  % Only Fed Fund shock
    simdout = simdatarec(xdata(beg_samp:end,:),nlag,nstep,hascon,trendval,vardetrend,fflocation,sortprog,ndraws);
end

disp('Backing IRF and VD for each simulated series')
for zx=1:ndraws;
    if rem(zx,10)==0, disp(['   series #' int2str(zx)]); end
    if (TTShock==1 && ISShock==0 && FFShock==0)
        [simimpzout(:,:,zx),simvdout(:,:,zx)] = mkimplr(simdout(:,:,zx),nlag,nstep,hascon,trendval,vardetrend,fflocation,sortprog,ISShock);
    elseif or(ISShock,TTShock)
        [simimpzout(:,:,:,zx),simvdout(:,:,:,zx)] = mkimplr(simdout(:,:,zx),nlag,nstep,hascon,trendval,vardetrend,fflocation,sortprog,ISShock);
    elseif FFShock == 1
        [simimpzout(:,:,:,zx),simvdout(:,:,:,zx)] = mkimprec(simdout(:,:,zx),nlag,nstep,hascon,trendval,vardetrend,fflocation,sortprog,ISShock);
    else
        error('no shock specified')
    end
end


%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
%       PLOTTING IRFs          %
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~% 
 
figrwz = ceil(size(impzout,2)/2); % Number of subplots

if or(ISShock,TTShock)  % Plot the first technology shock
    figure
    for zx=1:size(impzout,2);
        subplot(figrwz,2,zx)
        si = (squeeze(simimpzout(1:pnstep,zx,1,:)));
        mkpicgennew6(squeeze(impzout(1:pnstep,zx,1)),si)
        title(deblank(namz(zx,:)));
    end
    if ISShock == 1;
        suptitle('Investment Shock')
    else
        suptitle('Technology Shock')
    end
    set(gcf,'PaperPosition',[0.2500    0.2500    8.0000   10.5000])
end

if ISShock == 1;        % Plot the second technology shock
    figure
    for zx=1:size(impzout,2);
        subplot(figrwz,2,zx)
        si = (squeeze(simimpzout(1:pnstep,zx,2,:)));
        mkpicgennew6(squeeze(impzout(1:pnstep,zx,2)),si)
        title(deblank(namz(zx,:)));
    end
    suptitle('Technology Shock')
    set(gcf,'PaperPosition',[0.2500    0.2500    8.0000   10.5000])
end


if FFShock == 1;
    figure 
    for zx=1:size(impzout,2);
        %make it a cut of the FF rate. 
        subplot(figrwz,2,zx)
        si =  -1*(squeeze(simimpzout(1:pnstep,zx,fflocation,:)));
        mkpicgennew6(squeeze(-1*impzout(1:pnstep,zx,fflocation)),si)
        title(deblank(namz(zx,:)));
    end
    suptitle('Fed Funds Shock')
    set(gcf,'PaperPosition',[0.2500    0.2500    8.0000   10.5000])
end


%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
%     PLOTTING VARIANCE DECOMPOSITIONS     %
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%

nn=size(num2str(size(vdout,2)));
dumb=['1 '; '5 ' ;'10';num2str(size(vdout,2))];
if nn == 3
    dumb=['1  '; '5  ' ;'10 ';num2str(size(vdout,2))];
elseif nn == 4
    dumb=['1   '; '5   ' ;'10  ';num2str(size(vdout,2))];
end


if ISShock == 1;  
    disp('  '); disp('  Variance Due to Investment Specific Shock');
    tableMAT(namz,dumb,roundn(vdout(:,[1 5 10 end],1),-3) )
    
    disp('  '); disp('  Variance Due to Neutral Technology Shock');
    tableMAT(namz,dumb,roundn(vdout(:,[1 5 10 end],2),-3) )
    
    disp('  '); disp('  Variance Due to both Technology Shock')
    tableMAT(namz,dumb,roundn(vdout(:,[1 5 10 end],1)+vdout(:,[1 5 10 end],2),-3) )
elseif TTShock == 1
    disp('  '); disp('  Variance Due to Neutral Technology Shock');
    tableMAT(namz,dumb,roundn(vdout(:,[1 5 10 end],1),-3) )
end

if FFShock == 1; % Check it gives the right thing...   
    disp('  '); disp('   Variance Due to Monetary Policy Shock');
    tableMAT(namz,dumb,roundn(vdout(:,[1 5 10 end],fflocation),-3) )
end

