function [mcimpz,betazout,vout,outbbetavar,outsigur] = mkimplrmlebayesciconstwc(vardata,nlags,nstep,hasconst,ndraws,sortlev);
% mkimplrmlebayesciconstwc : Calculate Bayesian Confidence Intervals For
% Impulse Responses to a Long Run VAR
%  [mcimpz,betazout,vout,outbbetavar,outsigur] =
%  mkimplrmlebayesciconstwc(vardata,nlags,nstep,hasconst,ndraws,sortlev);
% REQUIRED INPUTS
% vardata: Date used for the VAR, each variable is in a column. It is assumed that that either labor productivity growth or output growth is listed first. 
% NLAGS is the number of lags in the VAR. 
% NSTEP is the number of steps out to calculate the impulse responses. 
% OPTIONAL INPUTS 
% HASCONST determines whether the VAR has a constant. 1 equals yes. 
% SORTLEV is the name of the file used to sort out the variables and put them in level
% in other words, you might estimate the var (dy dh) and   want impulse responses for 
% for Y and H. The program sortlew will describe how to change (dy dh) into (y h). 
% OUTPUT FROM THE PROGRAM
% mcimpz are the set of impulse responses they are a multidimensional matrix that the number of rows is the number of steps.
% the columns are the variables and the third dimension (the worksheet) are the different shocks. 
% betazout is the distribution of betas
% vout is the distribution of Variance Covariance Matricies. 
%By Robert Vigfusson August 2004


[nobs,nvars] = size(vardata);  %number of variables
 
y   = vardata((nlags+1):end,1:end);
xx  = zeros(length(y),nlags*nvars);
x   = zeros(length(y),nlags*nvars);
erz = zeros(size(y));

for zx = 1:nlags;
   xx(:,(zx-1)*nvars+1:zx*nvars)      = vardata((nlags+1+1)-zx:end-zx+1,:) -  vardata((nlags+1)-zx:end-zx,:);
   x(:,(zx-1)*nvars+1:zx*nvars)       = vardata((nlags+1)-zx:end-zx,:);
end;



if hasconst;
   const = ones(size(xx,1),1);
else
   const = [];
end;

 
 
 betavar = [const x]\y; 
     uzur = y - [const x]*betavar;
     betatmp = betavar;%(hasconst+1:end,:);
     betatmp = betatmp(:);
     betavar = betavar';
     sigur   = cov(uzur,1);
   invxpx = inv([const x]'*[const x]);
 
   outbbetavar = betavar;
   outsigur = sigur;
   
   
%      [W] = wishrnd(Sc,nu)   in stats toolbox
%  
%   W = returned random symmetric positive definite matrix
%   
%   Sc = p x p symmetric, postitive definite "scale" matrix 
%   nu = "degrees of freedom" (when integer)
%      = "number of observation" (when integer)
%      = precision parameter (when non-integer)

betazout = zeros(1+nlags*nvars,nvars,ndraws);
vout = zeros(nvars,nvars,ndraws);

for icnt = 1:ndraws;
 
    simw = inv(wishrnd(inv(nobs*sigur),nobs));
    simbetaz = betatmp+ chol(kron(simw,invxpx))'*randn(length(betatmp),1) ;
    simbetazwc = reshape(simbetaz,hasconst+nlags*nvars,nvars)';
    
    vout(:,:,icnt) = simw;
    betazout(:,:,icnt) =   simbetazwc';
    simbetaz =    simbetazwc(:,(hasconst+1):end);
    lrb1 = eye(nvars);
      for zax=1:nvars;
          lrb1(:,zax) = lrb1(:,zax)- sum(simbetaz(:,(zax):nvars:end),2);
      end;
       invlrb1 = inv(lrb1);
       anewmle=inv(lrb1*chol(invlrb1*simw*invlrb1')');
  
a0inv = inv(anewmle);
azeroout  = anewmle;
a0betazout = simbetaz;
betaz =simbetaz';
 
%calculate moving average representation and through this the impulse responses.
%in calculating moving average representation include the std of the shocks.
 
marep = zeros([nvars nvars nstep]);

for allshks = 1:size(erz,2);
   shock = [a0inv(:,allshks)';zeros(nstep-1,nvars)];  
   zlag=zeros(1,nlags*nvars);   
   for jj=1:nstep;
      imptmp = shock(jj,:)+zlag*betaz(1:length(betaz),:);
      marep(:,allshks,jj)=imptmp;
      zlag=[imptmp zlag(1,1:(nlags-1)*nvars)];
   end;
end;
 
  

if nargin<6;
   %do everything in differences
    impzout  = squeeze(marep(:,1,:))';
  
else 
   %report results in levels.
   nx = 1;
   malevtemp = feval(sortlev,squeeze(marep(:,nx,:))')';
   malev = zeros(size(malevtemp,1),nvars, size(malevtemp,2));
   malev(:,1,:) = malevtemp;
   for nx=2:size(marep,2);
      malev(:,nx,:) =  feval(sortlev,squeeze(marep(:,nx,:))')';
   end
   %impulse responses   
    impzout  = squeeze(malev(:,1,:))';
end;


mcimpz(:,:,icnt) = impzout;

end;

 
 